@@ -44,7 +44,9 @@ class CUPT_API File
 	 * @warning You must not use constructed object if @a error is not empty.
 	 *
 	 * @param path path to file or shell command, see @a mode
-	 * @param mode any value, accepted as @a mode in @c fopen(3), or @c "pr" - special value to treat @a path as shell pipe
+	 * @param mode any value, accepted as @a mode in @c fopen(3); or @c "pr" /
+	 *   @c "pw" - special values to treat @a path as shell pipe with an opened
+	 *   handle for reading / writing, respectively
 	 * @param [out] error if open fails, human readable error will be placed here
 	 */
 	File(const string& path, const char* mode, string& error);
@@ -125,6 +127,10 @@ class CUPT_API File
 	 * @param size size of the buffer
 	 */
 	void put(const char* data, size_t size);
+	/// @cond
+	void unbufferedPut(const char* data, size_t size);
+	/// @endcond
+
 
 	/// checks for the end of file condition
 	bool eof() const;
