@@ -1,5 +1,5 @@
 /**************************************************************************
-*   Copyright (C) 2010 by Eugene V. Lyubimkin                             *
+*   Copyright (C) 2010-2011 by Eugene V. Lyubimkin                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License                  *
@@ -998,7 +998,7 @@ bool PackagesWorker::__build_actions_graph(GraphAndAttributes& gaa)
 {
 	if (!__desired_state)
 	{
-		fatal("worker desired state is not given");
+		_logger->loggedFatal(Logger::Subsystem::Packages, 2, "worker desired state is not given");
 	}
 
 	bool debugging = _config->getBool("debug::worker");
@@ -1178,7 +1178,7 @@ void __build_mini_action_graph(const shared_ptr< const Cache >& cache,
 	__iterate_over_graph(*cache, miniGaa, true, debugging);
 }
 
-void __split_heterogeneous_actions(const shared_ptr< const Cache >& cache,
+void __split_heterogeneous_actions(const shared_ptr< const Cache >& cache, Logger& logger,
 		vector< InnerActionGroup >& actionGroups, GraphAndAttributes& gaa,
 		GraphAndAttributes::Attribute::Level level, bool debugging)
 {
@@ -1205,7 +1205,8 @@ void __split_heterogeneous_actions(const shared_ptr< const Cache >& cache,
 				{
 					actionStrings.push_back(it->toString());
 				}
-				fatal("internal error: unable to schedule circular actions '%s'", join(", ", actionStrings).c_str());
+				logger.loggedFatal(Logger::Subsystem::Packages, 2,
+						format2("internal error: unable to schedule circular actions '%s'", join(", ", actionStrings)));
 			}
 
 			// we build a mini-graph with reduced number of edges
@@ -1239,8 +1240,9 @@ void __split_heterogeneous_actions(const shared_ptr< const Cache >& cache,
 								actionSubgroup.dpkgFlags.insert("--force-breaks");
 								break;
 							default:
-								fatal("internal error: worker: a relation '%s' cannot be soft",
-										BinaryVersion::RelationTypes::rawStrings[*removedRelationIt]);
+								logger.loggedFatal(Logger::Subsystem::Packages, 2,
+										format2("internal error: worker: a relation '%s' cannot be soft",
+										BinaryVersion::RelationTypes::rawStrings[*removedRelationIt]));
 						}
 					}
 				}
@@ -1255,7 +1257,7 @@ void __split_heterogeneous_actions(const shared_ptr< const Cache >& cache,
 				actionSubgroupsSorted.rbegin()->continued = false;
 			}
 
-			__split_heterogeneous_actions(cache, actionSubgroupsSorted, miniGaa,
+			__split_heterogeneous_actions(cache, logger, actionSubgroupsSorted, miniGaa,
 					Attribute::Level((int)level+1), debugging);
 
 			FORIT(actionSubgroupIt, actionSubgroupsSorted)
@@ -1290,13 +1292,13 @@ void __split_heterogeneous_actions(const shared_ptr< const Cache >& cache,
 	newActionGroups.swap(actionGroups);
 }
 
-static string __get_codename_and_component_string(const Version& version)
+static string __get_codename_and_component_string(const Version& version, const string& baseUri)
 {
 	vector< string > parts;
 	FORIT(sourceIt, version.sources)
 	{
 		auto releaseInfo = sourceIt->release;
-		if (releaseInfo->baseUri.empty())
+		if (releaseInfo->baseUri != baseUri)
 		{
 			continue;
 		}
@@ -1305,112 +1307,126 @@ static string __get_codename_and_component_string(const Version& version)
 	return join(",", parts);
 }
 
+static string __get_long_alias_tail(const Version& version, const string& baseUri)
+{
+	return format2("%s %s %s", __get_codename_and_component_string(version, baseUri),
+			version.packageName, version.versionString);
+}
+
 map< string, pair< download::Manager::DownloadEntity, string > > PackagesWorker::__prepare_downloads()
 {
-	auto archivesDirectory = _get_archives_directory();
+	_logger->log(Logger::Subsystem::Packages, 2, "preparing downloads");
 
-	if (!_config->getBool("cupt::worker::simulate"))
+	map< string, pair< download::Manager::DownloadEntity, string > > downloads;
+	try
 	{
-		string partialDirectory = archivesDirectory + partialDirectorySuffix;
-		if (! fs::dirExists(partialDirectory))
+		auto archivesDirectory = _get_archives_directory();
+
+		if (!_config->getBool("cupt::worker::simulate"))
 		{
-			if (mkdir(partialDirectory.c_str(), 0755) == -1)
+			string partialDirectory = archivesDirectory + partialDirectorySuffix;
+			if (! fs::dirExists(partialDirectory))
 			{
-				fatal("unable to create partial directory '%s': EEE", partialDirectory.c_str());
+				if (mkdir(partialDirectory.c_str(), 0755) == -1)
+				{
+					_logger->loggedFatal(Logger::Subsystem::Packages, 3,
+							format2e("unable to create partial directory '%s'", partialDirectory));
+				}
+			}
+		}
+
+
+		DebdeltaHelper debdeltaHelper;
+
+		static const vector< Action::Type > actions = { Action::Install, Action::Upgrade, Action::Downgrade };
+		FORIT(actionIt, actions)
+		{
+			const Resolver::SuggestedPackages& suggestedPackages = __actions_preview->groups[*actionIt];
+			FORIT(it, suggestedPackages)
+			{
+				const shared_ptr< const BinaryVersion >& version = it->second.version;
+
+				const string& packageName = version->packageName;
+				const string& versionString = version->versionString;
+
+				auto downloadInfo = version->getDownloadInfo();
+
+				// we need at least one real URI
+				if (downloadInfo.empty())
+				{
+					_logger->loggedFatal(Logger::Subsystem::Packages, 3,
+							format2("no available download URIs for %s %s", packageName, versionString));
+				}
+
+				// paths
+				auto basename = _get_archive_basename(version);
+				auto downloadPath = archivesDirectory + partialDirectorySuffix + '/' + basename;
+				auto targetPath = archivesDirectory + '/' + basename;
+
+				// exclude from downloading packages that are already present
+				if (fs::fileExists(targetPath) &&
+					version->file.hashSums.verify(targetPath))
+				{
+					continue;
+				}
+
+				download::Manager::DownloadEntity downloadEntity;
+
+				FORIT(it, downloadInfo)
+				{
+					string uri = it->baseUri + '/' + it->directory + '/' + version->file.name;
+
+					string shortAlias = packageName;
+					string longAlias = it->baseUri + ' ' + __get_long_alias_tail(*version, it->baseUri);
+
+					downloadEntity.extendedUris.push_back(
+							download::Manager::ExtendedUri(uri, shortAlias, longAlias));
+				}
+				{
+					auto debdeltaDownloadInfo = debdeltaHelper.getDownloadInfo(version, _cache);
+					FORIT(it, debdeltaDownloadInfo)
+					{
+						const string& uri = it->uri;
+						string longAlias = it->baseUri + ' ' + __get_long_alias_tail(*version, it->baseUri);
+
+						downloadEntity.extendedUris.push_back(
+								download::Manager::ExtendedUri(uri, packageName, longAlias));
+					}
+				}
+
+				// caution: targetPath and downloadEntity.targetPath are different
+				downloadEntity.targetPath = downloadPath;
+				downloadEntity.size = version->file.size;
+
+				downloadEntity.postAction = [version, downloadPath, targetPath]() -> string
+				{
+					if (!fs::fileExists(downloadPath))
+					{
+						return __("unable to find downloaded file");
+					}
+					if (!version->file.hashSums.verify(downloadPath))
+					{
+						unlink(downloadPath.c_str()); // intentionally ignore errors if any
+						return __("hash sums mismatch");
+					}
+
+					return fs::move(downloadPath, targetPath);
+				};
+
+				auto downloadValue = std::make_pair(std::move(downloadEntity), targetPath);
+				downloads.insert(std::make_pair(packageName, downloadValue));
 			}
 		}
 	}
-
-	map< string, pair< download::Manager::DownloadEntity, string > > downloads;
-
-	DebdeltaHelper debdeltaHelper;
-
-	static const vector< Action::Type > actions = { Action::Install, Action::Upgrade, Action::Downgrade };
-	FORIT(actionIt, actions)
+	catch (...)
 	{
-		const Resolver::SuggestedPackages& suggestedPackages = __actions_preview->groups[*actionIt];
-		FORIT(it, suggestedPackages)
-		{
-			const shared_ptr< const BinaryVersion >& version = it->second.version;
-
-			const string& packageName = version->packageName;
-			const string& versionString = version->versionString;
-
-			auto downloadInfo = version->getDownloadInfo();
-
-			// we need at least one real URI
-			if (downloadInfo.empty())
-			{
-				fatal("no available download URIs for %s %s", packageName.c_str(), versionString.c_str());
-			}
-
-			// paths
-			auto basename = _get_archive_basename(version);
-			auto downloadPath = archivesDirectory + partialDirectorySuffix + '/' + basename;
-			auto targetPath = archivesDirectory + '/' + basename;
-
-			// exclude from downloading packages that are already present
-			if (fs::fileExists(targetPath) &&
-				version->file.hashSums.verify(targetPath))
-			{
-				continue;
-			}
-
-			download::Manager::DownloadEntity downloadEntity;
-
-			string longAliasTail = sf("%s %s %s", __get_codename_and_component_string(*version).c_str(),
-						packageName.c_str(), versionString.c_str());
-			FORIT(it, downloadInfo)
-			{
-				string uri = it->baseUri + '/' + it->directory + '/' + version->file.name;
-
-
-				string shortAlias = packageName;
-				string longAlias = it->baseUri + ' ' + longAliasTail;
-
-				downloadEntity.extendedUris.push_back(
-						download::Manager::ExtendedUri(uri, shortAlias, longAlias));
-			}
-			{
-				auto debdeltaDownloadInfo = debdeltaHelper.getDownloadInfo(version, _cache);
-				FORIT(it, debdeltaDownloadInfo)
-				{
-					const string& uri = it->uri;
-					string longAlias = it->baseUri + ' ' + longAliasTail;
-
-					downloadEntity.extendedUris.push_back(
-							download::Manager::ExtendedUri(uri, packageName, longAlias));
-				}
-			}
-
-			// caution: targetPath and downloadEntity.targetPath are different
-			downloadEntity.targetPath = downloadPath;
-			downloadEntity.size = version->file.size;
-
-			downloadEntity.postAction = [version, downloadPath, targetPath]() -> string
-			{
-				if (!fs::fileExists(downloadPath))
-				{
-					return __("unable to find downloaded file");
-				}
-				if (!version->file.hashSums.verify(downloadPath))
-				{
-					unlink(downloadPath.c_str()); // intentionally ignore errors if any
-					return __("hash sums mismatch");
-				}
-
-				return fs::move(downloadPath, targetPath);
-			};
-
-			auto downloadValue = std::make_pair(std::move(downloadEntity), targetPath);
-			downloads.insert(std::make_pair(packageName, downloadValue));
-		}
+		_logger->loggedFatal(Logger::Subsystem::Packages, 2, "failed to prepare downloads");
 	}
 
 	return downloads;
 }
 
-vector< Changeset > __split_action_groups_into_changesets(
+vector< Changeset > __split_action_groups_into_changesets(Logger& logger,
 		const vector< InnerActionGroup >& actionGroups,
 		const map< string, pair< download::Manager::DownloadEntity, string > >& downloads)
 {
@@ -1458,8 +1474,8 @@ vector< Changeset > __split_action_groups_into_changesets(
 		vector< string > unconfiguredPackageNames;
 		std::copy(unpackedPackageNames.begin(), unpackedPackageNames.end(),
 				std::back_inserter(unconfiguredPackageNames));
-		fatal("internal error: packages stay unconfigured: '%s'",
-				join(" ", unconfiguredPackageNames).c_str());
+		logger.loggedFatal(Logger::Subsystem::Packages, 2,
+				format2("internal error: packages stay unconfigured: '%s'", join(" ", unconfiguredPackageNames)));
 	}
 
 	return result;
@@ -1505,7 +1521,7 @@ size_t __get_max_download_amount(const vector< Changeset >& changesets, bool deb
 }
 
 void __set_force_options_for_removals_if_needed(const Cache& cache,
-		vector< InnerActionGroup >& actionGroups)
+		Logger& logger, vector< InnerActionGroup >& actionGroups)
 {
 	auto systemState = cache.getSystemState();
 	FORIT(actionGroupIt, actionGroups)
@@ -1533,9 +1549,10 @@ void __set_force_options_for_removals_if_needed(const Cache& cache,
 					auto installedRecord = systemState->getInstalledInfo(packageName);
 					if (!installedRecord)
 					{
-						fatal("internal error: worker: __set_force_options_for_removals_if_needed: "
+						logger.loggedFatal(Logger::Subsystem::Packages, 2,
+								format2("internal error: worker: __set_force_options_for_removals_if_needed: "
 								"there is no installed record for the package '%s' which is to be removed",
-								packageName.c_str());
+								packageName));
 					}
 					typedef system::State::InstalledRecord::Flag IRFlag;
 					if (installedRecord->flag == IRFlag::Reinstreq || installedRecord->flag == IRFlag::HoldAndReinstreq)
@@ -1558,7 +1575,7 @@ void __set_force_options_for_removals_if_needed(const Cache& cache,
 	}
 }
 
-void __get_action_groups(const shared_ptr< const Cache >& cache,
+void __get_action_groups(const shared_ptr< const Cache >& cache, Logger& logger,
 		GraphAndAttributes& gaa, vector< InnerActionGroup >* actionGroupsPtr,
 		bool debugging)
 {
@@ -1570,9 +1587,9 @@ void __get_action_groups(const shared_ptr< const Cache >& cache,
 
 	typedef GraphAndAttributes::Attribute Attribute;
 	auto initialSplitLevel = Attribute::FromVirtual;
-	__split_heterogeneous_actions(cache, *actionGroupsPtr, gaa, initialSplitLevel, debugging);
+	__split_heterogeneous_actions(cache, logger, *actionGroupsPtr, gaa, initialSplitLevel, debugging);
 
-	__set_force_options_for_removals_if_needed(*cache, *actionGroupsPtr);
+	__set_force_options_for_removals_if_needed(*cache, logger, *actionGroupsPtr);
 }
 
 vector< Changeset > PackagesWorker::__get_changesets(GraphAndAttributes& gaa,
@@ -1584,8 +1601,8 @@ vector< Changeset > PackagesWorker::__get_changesets(GraphAndAttributes& gaa,
 	vector< Changeset > changesets;
 	{
 		vector< InnerActionGroup > actionGroups;
-		__get_action_groups(_cache, gaa, &actionGroups, debugging);
-		changesets = __split_action_groups_into_changesets(actionGroups, downloads);
+		__get_action_groups(_cache, *_logger, gaa, &actionGroups, debugging);
+		changesets = __split_action_groups_into_changesets(*_logger, actionGroups, downloads);
 	}
 
 	if (archivesSpaceLimit)
@@ -1599,8 +1616,9 @@ vector< Changeset > PackagesWorker::__get_changesets(GraphAndAttributes& gaa,
 		if (maxDownloadAmount > archivesSpaceLimit)
 		{
 			// we failed to fit in limit
-			fatal("unable to fit in archives space limit '%zu', best try is '%zu'",
-					archivesSpaceLimit, maxDownloadAmount);
+			_logger->loggedFatal(Logger::Subsystem::Packages, 3,
+					format2("unable to fit in archives space limit '%zu', best try is '%zu'",
+					archivesSpaceLimit, maxDownloadAmount));
 		}
 	}
 
@@ -1639,30 +1657,39 @@ vector< Changeset > PackagesWorker::__get_changesets(GraphAndAttributes& gaa,
 
 void PackagesWorker::__run_dpkg_command(const string& flavor, const string& command, const string& commandInput)
 {
-	auto errorId = sf(__("dpkg '%s' action '%s'"), flavor.c_str(), command.c_str());
+	auto errorId = format2(__("dpkg '%s' action '%s'"), flavor, command);
 	_run_external_command(Logger::Subsystem::Packages, command, commandInput, errorId);
 }
 
 void PackagesWorker::__clean_downloads(const Changeset& changeset)
 {
-	internal::Lock archivesLock(_config, _get_archives_directory() + "/lock");
-
-	bool simulating = _config->getBool("cupt::worker::simulate");
-	FORIT(it, changeset.downloads)
+	_logger->log(Logger::Subsystem::Packages, 2, "cleaning downloaded archives");
+	try
 	{
-		const char* targetPath = it->second.c_str();
-		if (simulating)
+		internal::Lock archivesLock(_config, _get_archives_directory() + "/lock");
+
+		bool simulating = _config->getBool("cupt::worker::simulate");
+		FORIT(it, changeset.downloads)
 		{
-			simulate("removing archive '%s'", targetPath);
-		}
-		else
-		{
-			if (unlink(targetPath) == -1)
+			const string& targetPath = it->second;
+			if (simulating)
 			{
-				fatal("unable to remove file '%s': EEE", targetPath);
+				simulate2("removing archive '%s'", targetPath);
+			}
+			else
+			{
+				if (unlink(targetPath.c_str()) == -1)
+				{
+					_logger->loggedFatal(Logger::Subsystem::Packages, 3,
+							format2e("unable to remove file '%s'", targetPath));
+				}
 			}
 		}
 	}
+	catch (...)
+	{
+		_logger->loggedFatal(Logger::Subsystem::Packages, 2, "failed to clean downloaded archives");
+	}
 }
 
 void PackagesWorker::__do_dpkg_pre_actions()
@@ -1778,8 +1805,8 @@ string PackagesWorker::__generate_input_for_preinstall_v2_hooks(
 				}
 			}
 
-			result += sf("%s %s %s %s %s\n", packageName.c_str(), oldVersionString.c_str(),
-					compareVersionStringsSign.c_str(), newVersionString.c_str(), path.c_str());
+			result += format2("%s %s %s %s %s\n", packageName, oldVersionString,
+					compareVersionStringsSign, newVersionString, path);
 		}
 	}
 
@@ -1877,8 +1904,8 @@ void PackagesWorker::markAsAutomaticallyInstalled(const string& packageName, boo
 	auto simulating = _config->getBool("cupt::worker::simulate");
 
 	{ // logging
-		auto message = sf("marking '%s' as %s installed",
-				packageName.c_str(), targetStatus ? "automatically" : "manually");
+		auto message = format2("marking '%s' as %s installed",
+				packageName, targetStatus ? "automatically" : "manually");
 		_logger->log(Logger::Subsystem::Packages, 2, message);
 	}
 
@@ -1886,40 +1913,50 @@ void PackagesWorker::markAsAutomaticallyInstalled(const string& packageName, boo
 	{
 		string prefix = targetStatus ?
 					__("marking as automatically installed") : __("marking as manually installed");
-		simulate("%s: %s", prefix.c_str(), packageName.c_str());
+		simulate2("%s: %s", prefix, packageName);
 	}
 	else
 	{
-		if (targetStatus)
+		try
 		{
-			__auto_installed_package_names.insert(packageName);
-		}
-		else
-		{
-			__auto_installed_package_names.erase(packageName);
-		}
-		auto extendedInfoPath = _cache->getPathOfExtendedStates();
-		auto tempPath = extendedInfoPath + ".cupt.tmp";
-
-		{
-			string errorString;
-			File tempFile(tempPath, "w", errorString);
-			if (!errorString.empty())
+			if (targetStatus)
 			{
-				fatal("unable to open temporary file '%s': %s", tempPath.c_str(), errorString.c_str());
+				__auto_installed_package_names.insert(packageName);
+			}
+			else
+			{
+				__auto_installed_package_names.erase(packageName);
+			}
+			auto extendedInfoPath = _cache->getPathOfExtendedStates();
+			auto tempPath = extendedInfoPath + ".cupt.tmp";
+
+			{
+				string errorString;
+				File tempFile(tempPath, "w", errorString);
+				if (!errorString.empty())
+				{
+					_logger->loggedFatal(Logger::Subsystem::Packages, 3,
+							format2("unable to open temporary file '%s': %s", tempPath, errorString));
+				}
+
+				// filling new info
+				FORIT(packageNameIt, __auto_installed_package_names)
+				{
+					tempFile.put(format2("Package: %s\nAuto-Installed: 1\n\n", *packageNameIt));
+				}
 			}
 
-			// filling new info
-			FORIT(packageNameIt, __auto_installed_package_names)
+			auto moveResult = fs::move(tempPath, extendedInfoPath);
+			if (!moveResult.empty())
 			{
-				tempFile.put(sf("Package: %s\nAuto-Installed: 1\n\n", packageNameIt->c_str()));
+				_logger->loggedFatal(Logger::Subsystem::Packages, 3,
+						format2("unable to renew extended states file: %s", moveResult));
 			}
 		}
-
-		auto moveResult = fs::move(tempPath, extendedInfoPath);
-		if (!moveResult.empty())
+		catch (...)
 		{
-			fatal("unable to renew extended states file: %s", moveResult.c_str());
+			_logger->loggedFatal(Logger::Subsystem::Packages, 2,
+					"failed to change the 'automatically installed' flag");
 		}
 	}
 }
@@ -1958,7 +1995,7 @@ void PackagesWorker::__do_downloads(const vector< pair< download::Manager::Downl
 
 		if (!downloadResult.empty())
 		{
-			fatal("there were download errors");
+			_logger->loggedFatal(Logger::Subsystem::Packages, 2, "there were download errors");
 		}
 	}
 }
